#define version "40_charlie"
//**********************************************************
// BIBLIOTHEQUES
// incluses dans le package Arduino de base

// ajoutées
// *** GENERAL *** formatage des entrees sorties ASCII
#include <Streaming.h>
// *** TEMPS ***
#include <Time.h>

//**********************************************************
// DECLARATIONS
// *** GENERAL *** DEBUG : 0=pas de debug, valeur selon besoins
int debug=1;

// entrées-sorties
int led=13;

// entrées-sorties
int charlie_led[4] = {5,7,10,11};    // pour charlieplexage 4 sorties = 12 leds
int charlie_i;			     // indice de multiplexage
char charlie_tour='0',charlie_tt='3';       // comptage tours pour intensité

// le tableau magique : 0=0volts(LOW) 1=5volts(HIGH) 2=haute_impédance(Input)
struct charlie_t { int etat[4]; } ;
struct charlie_t charliet[] = {
	{0,1,2,2},    // R1
	{0,2,1,2},    // V1
	{0,2,2,1},    // B1

	{2,0,1,2},    // R2
	{2,0,2,1},    // V2
	{1,0,2,2},    // B2

	{2,2,0,1},    // R3
	{1,2,0,2},    // V3
	{2,1,0,2},    // B3

	{1,2,2,0},    // R4
	{2,1,2,0},    // V4
	{2,2,1,0},    // B4

	{2,2,2,2}     // tout éteint
};
//                 111222333444
//                 RVBRVBRVBRVB
char* charlie_map="999999999999";	// variable de commande
// pour les 12 positions : ' '=éteinte 'R,V,B'=allumée (n'importe quel caractère autre que ' ') 
// pour les 12 positions : ' ','0'=éteinte '1...9'=allumée 
char* charlie_mapv="000000000000";
char charlie_unite='0';

// temporisations
int tempo_a=0,tempo_b=0,tempo_led=0;
// vitesse de clignotement : en millisecondes
int tempo_a_reset=500;
int tempo_b_reset=12000;
int tempo_led_reset=1;

// gestion du temps, clavier, sp
#include "decl_sp.h"

// *** GENERAL *** 
// speedomètre = calcul de vitesse ...
unsigned long tours,tours_ps,tours_led;

// GRAFCETS
int grafcet_a=1, grafcet_av=-1, itest=0;

//--------------------------------
// variables très locales !
int i,j,k;
int bp,bpm;

//**********************************************************
// SOUS-PROGRAMMES
// gestion du temps, formatages, gestion clavier
#include "sp.h"

//**********************************************************
// INITIALISATIONS
void setup() {
// initialize the pins
  pinMode(led, OUTPUT);
  pinMode(2, INPUT);
/*
  pinMode(led1, OUTPUT);
  pinMode(led2, OUTPUT);
  pinMode(led3, OUTPUT);
  pinMode(led4, OUTPUT);
*/
// mise à l'heure
  setTime(0,0,0,21,04,13); // 20h01 le 25 déc 2013 setupArduinoTime(2013,25,21,12,15,00));

// ligne série (pour traces)
  Serial.begin(9600);  // ou 9600 selon votre port COM
  Serial << "Serial ok" << endl;

  } // Fin du SETUP

//**********************************************************
// PROGRAMME PRINCIPAL
void loop() {

// compte-tours par seconde et led_a "heart-beat" commutée
  tours++; tours_led++;
  if (tours_led > 10000) { tours_led=0; digitalWrite(led,!digitalRead(led));}

// gestion du temps (dont temporisations)
  ger_top_centieme ();

// opérations à chaque millième de seconde
  if (top_1 > 0) {
    if (tempo_led > top_1 ) { tempo_led -= top_1; } else { tempo_led = 0; }
    top_1 = 0;
  }

// opérations à chaque centième de seconde
  if (top_10 > 0) {
// gestion des TEMPORISATIONS (en ms mais précise à 10ms près)
    if (tempo_a > top_10 ) { tempo_a -= top_10; } else { tempo_a = 0; }
    if (tempo_b > top_10 ) { tempo_b -= top_10; } else { tempo_b = 0; }
//    if (tempo_led > top_10 ) { tempo_led -= top_10; } else { tempo_led = 0; }
    top_10 = 0;
  }
 
/*
// opérations à chaque dixième de seconde
  if (top_100ms) {
    top_100ms = false;
//    if (tempo_XX >0) tempo_XX--;
  }
 
// opérations à chaque 1/4 de seconde
  if (top_250ms) {
    top_250ms = false;
  }
*/

// opérations à chaque seconde
  if (top_1000ms) {
    tours_ps=tours; tours=0;
    top_1000ms = false;
  }

//--------------------------------
// chaque minute :
   if (top_60000ms) {
        top_60000ms=false;
   }  // fin minute

//--------------------------------********************--------------------------------
// trace GRAFCET A
//if (grafcet_a != grafcet_av || strcmp(charlie_map,charlie_mapv) !=0) {
if (strcmp(charlie_map,charlie_mapv) !=0) {
  Serial << grafcet_a << "*=" << charlie_map << "=" << endl;
  strncpy(charlie_mapv,charlie_map,12);
}

switch (grafcet_a) {
//--------------------------------
// test dimmer
  case 1:
    if (tempo_a==0) {
//                 RVBrvbRVBrvb
      memset (charlie_map,'0',12);  // RAZ du buffer
      tempo_a = tempo_a_reset;
      itest=11;
      grafcet_a++;
      Serial << "Test dimmer " << tempo_a << endl;
    }
  break;
  case 2:
    if (tempo_a==0) {
      charlie_unite++;
      if (charlie_unite > charlie_tt || charlie_unite < '0') charlie_unite='0';
      charlie_map[0]=charlie_unite;
      charlie_map[4]=charlie_unite;
      charlie_map[8]=charlie_unite;
      charlie_map[9]=charlie_unite;
      charlie_map[10]=charlie_unite;
      charlie_map[11]=charlie_unite;
      tempo_a = tempo_a_reset;
      itest--;
      if (itest == 0) { itest=24; grafcet_a++;}
    }
  break;

  case 3:
    if (tempo_a==0) {
      charlie_unite++;
      if (charlie_unite > charlie_tt || charlie_unite < '0') charlie_unite='0';
      charlie_map[0]=charlie_unite-1;
      charlie_map[3]=charlie_unite;
      charlie_map[6]=charlie_unite+1;
      charlie_map[9]=charlie_unite+2;
      tempo_a = tempo_a_reset;
      itest--;
      if (itest == 0) { itest=24; grafcet_a--;}
    }
  break;

// test 12 leds seules
  case 21:
    if (tempo_a==0) {
//                 RVBrvbRVBrvb
      memset (charlie_map,' ',12);  // RAZ du buffer
      tempo_a = tempo_a_reset;
      itest=11;
      grafcet_a++;
      Serial << "Test 12 leds" << endl;
    }
  break;
  case 22:
    if (tempo_a==0) {
      charlie_map[itest]=' ';
      itest++;
      if (itest>11) itest=0;
      charlie_map[itest]='1';
      tempo_a = tempo_a_reset;
      if (itest==11) grafcet_a++;
    }
  break;

// test 7 couleurs sur les 4 leds
  case 3:
    memset (charlie_map,' ',12);  // RAZ du buffer
    itest=0;
    grafcet_a++;
    Serial << "Test 7 couleurs" << endl;
  break;
  case 4:
//                 012345678901
//                 RVBrvbRVBrvb
    if (tempo_a==0) {
      k=itest & 1;
      for (j=0;j<4;j++) {if (k>0) { charlie_map[3*j] = '1';} else {charlie_map[3*j] = ' ';}}
      k=itest & 2;
      for (j=0;j<4;j++) {if (k>0) { charlie_map[3*j+1] = '1';} else {charlie_map[3*j+1] = ' ';}}
      k=itest & 4;
      for (j=0;j<4;j++) {if (k>0) { charlie_map[3*j+2] = '1';} else {charlie_map[3*j+2] = ' ';}}
      itest++;
      tempo_a = tempo_a_reset;
      if (itest > 40) grafcet_a++;
    }
  break;

// LIBRE
  case 5:
   grafcet_a=11;
   break;

// Hazard toutes
  case 11:
    memset (charlie_map,' ',12);  // RAZ du buffer
    randomSeed(analogRead(3));
    grafcet_a++;
    tempo_b=tempo_b_reset;
    Serial << "Hasard homogene" << endl;
  break;
  case 12:
    if (tempo_a==0) {
      i = random(0,3); 
      if ( charlie_map[i] == ' ') {
       for (j=0;j<4;j++) charlie_map[i+3*j] = '2';
      } else {
       for (j=0;j<4;j++) charlie_map[i+3*j] = ' ';
      }
      tempo_a = tempo_a_reset/4;
//    Serial << "(12)" << second() << "." << milliseconde << "=" << charlie_map << "=" << endl;
    }
    if (tempo_b==0) {
      grafcet_a++;
    }
  break;

// Hazard hasard
  case 13:
    memset (charlie_map,' ',12);  // RAZ du buffer
    randomSeed(analogRead(3));
    grafcet_a++;
    tempo_b=tempo_b_reset;
    Serial << "Hasard + decalage" << endl;
  break;
  case 14:
    if (tempo_a==0) {
// decalage chenillard
      for (i=0;i<3;i++) {
        for (j=9;j>2;j-=3) charlie_map[i+j] = charlie_map[i+j-3];
      }
// tirage couleur au hasard
      i = random(0,3);
      if ( charlie_map[i] == ' ') {
        charlie_map[i] = '4';
      } else {
        charlie_map[i] = ' ';
      }
      tempo_a = tempo_a_reset/4;
    }
    if (tempo_b==0) {
      grafcet_a++;
    }
  break;

// bouclage
  default:
    grafcet_a=1;
  break;
}

//--------------------------------
//--------------------------------********************--------------------------------
// CHARLIPLEXAGE
/*
// TOUT OU RIEN
  if (tempo_led==0) {
// recherche led suivante à allumer
      i=0;
      do  {
         i++; 
         charlie_i++;  if (charlie_i > 11) charlie_i=0;
      } while ( (charlie_map[charlie_i] == ' ' || charlie_map[charlie_i] == '0') && i < 12);
// led à allumer
      if (charlie_map[charlie_i] != ' ') {
        charlie(charlie_i);
//        Serial << i << "#" << charlie_i << "{" << charlie_map << "}" << endl;
// extinction générale
      } else {
        charlie(12);
      }

      tempo_led = tempo_led_reset; 
    }    // tempo_led==0
*/

/*
// 10 fois 12
    if (tempo_led==0) {
//      Serial << "CHARLIE_i=" << charlie_i << endl;
      charlie_i++;  
      if (charlie_i > 11) {
        charlie_i=0;
// dimmer      
        charlie_tour--;
        if (charlie_tour < '0' ) charlie_tour=charlie_tt;
//         Serial << "charlie_tour=" << charlie_tour << " {" << charlie_map << "}" << endl;
      }
//        Serial << "##" << charlie_i << "{" << charlie_map << "}" << endl;

      if (charlie_map[charlie_i] > charlie_tour) {
        charlie(charlie_i);
//        Serial << "charlie_i=" << charlie_i << "charlie_tour=" << charlie_tour << "{" << charlie_map << "}" << endl;
// extinction générale
      } else {
        charlie(12);
      }
//        Serial << "###" << charlie_i << "{" << charlie_map << "}" << endl;

      tempo_led = tempo_led_reset; 
    }    // tempo_led==0
*/

// optimisé
    if (tempo_led==0) {
// recherche led suivante à allumer
      i=0;
      do  {
         i++; 
         charlie_i++;  
         if (charlie_i > 11) {
           charlie_i=0;
           charlie_tour--;
           if (charlie_tour < '0' ) charlie_tour=charlie_tt;
         }
       } while ( (charlie_map[charlie_i] <= charlie_tour) && i < 12);

      if (charlie_map[charlie_i] > charlie_tour) {
        charlie(charlie_i);
//        Serial << "charlie_i=" << charlie_i << "charlie_tour=" << charlie_tour << "{" << charlie_map << "}" << endl;
// extinction générale
      } else {
        charlie(12);
      }
//        Serial << "###" << charlie_i << "{" << charlie_map << "}" << endl;

      tempo_led = tempo_led_reset; 
    }    // tempo_led==0
//--------------------------------********************--------------------------------
}
//--------------------------------********************--------------------------------
// Sous-programmes
//--------------------------------********************-l-------------------------------
// commande de CHARLIPLEXAGE
void charlie(unsigned int ipos) {
    int il;
// pour éviter les aléas (micro-allumages) toutes sorties à OFF et/ou pullup OFF 
    for (il=0;il<4;il++) digitalWrite (charlie_led[il],LOW); 
// selon etat des sorties
    for (il=0;il<4;il++) {
		switch (charliet[ipos].etat[il]) {
// O volt
        case 0:
		  pinMode(charlie_led[il], OUTPUT); digitalWrite (charlie_led[il],LOW);
		  break;
// 5 volts
	case 1:
		  pinMode(charlie_led[il], OUTPUT); digitalWrite (charlie_led[il],HIGH);
		  break;
// haute impédance = entrée (dont le cas il=12 qui est "tout étetint")
        default:
	case 2:
	          pinMode(charlie_led[il], INPUT); // pullup OFF : digitalWrite (charlie_led[il],LOW); 
		  break;
		}
   }
}
//--------------------------------********************--------------------------------

